class Base():
    def __init__(self, client):
        self.client = client
        self.base_url = client.base_url
        self.short_url = client.short_url

        self.settings = client.settings

    def _make_request(
            self,
            endpoint,
            payload=None,
            params=None,
            method='post',
            files=None
    ):
        url = f'{self.base_url}/{endpoint}'

        if method.lower() == 'get':
            return self.client.session.get(url, params=params, verify=False)
        else:
            return self.client.session.post(
                url, json=payload, params=params, files=files, verify=False
            )



import configparser


class Settings:
    def __init__(self, server, config_file):
        self.config = configparser.ConfigParser()
        self.config_file = config_file
        self.config.server = server
        self.load_settings()

    def load_settings(self):
        self.config.read(self.config_file)

    def get(self, section, key, fallback=None):
        return self.config.get(section, key, fallback=fallback)

    def write(self):
        with open(self.config_file, 'w') as configfile:
            self.config.write(configfile)

    def set(self, section, key, value):
        if not self.config.has_section(section):
            self.config.add_section(section)
        self.config.set(section, key, str(value))

    def save(self, section, key, value):
        self.set(section, key, value)
        self.write()

    @property
    def server(self):
        return self.get('API', 'server')

    @property
    def offset(self):
        return self.get('API', 'offset')

    @property
    def count(self):
        return self.get('API', 'count')

    @property
    def logging_level(self):
        return self.get('LOGGING', 'level')

    @property
    def debug_mode(self):
        return self.get('LOGGING', 'debug_mode')



from typing import List
from .lib import Base


class Polls(Base):
    def create(
            self,
            answers: List[str],
            question: str,
            room_id: str,
            multiple_choice: bool = False,
            public_voters: bool = True
    ):
        return self._make_request('polls.create', payload={
            'answers': answers,
            'question': question,
            'roomId': room_id,
            'multipleChoice': multiple_choice,
            'publicVoters': public_voters,
        })

    def vote(self, msg_id: str, options: List[str]):
        return self._make_request('polls.vote', payload={
            'msgId': msg_id,
            'options': options
        })

    def get_results(self, msg_id: str):
        return self._make_request(
            'polls.results', params={'msgId': msg_id}, method='get'
        )

    def get_votes(self, msg_id: str):
        return self._make_request(
            'polls.votes', params={'msgId': msg_id}, method='get'
        )

    def retract(self, msg_id: str):
        return self._make_request(
            'polls.retractVote',
            payload={'msgId': msg_id}
        )



from .lib import Base


class Threads(Base):
    def create(self, message_id: str, room_id: str):
        return self._make_request(
            'thread.createOrJoinWithMessage', payload={
                'mid': message_id,
                'roomId': room_id,
            }
        )



"""
ECSS Chat Client - Python клиент для ECSS Chat
"""

from .auth import Auth
from .avatar import Avatar
from .chat import Chat
from .config import Settings
from .different import Different
from .dm_im import DmIm
from .folders import Folders
from .groups import Groups
from .polls import Polls
from .rooms import Rooms
from .supergroups import SuperGroups
from .testing import Testing
from .threads import Threads
from .users import Users


class Client:
    def __init__(self, server, username, password):
        self.username = username
        self.password = password
        self.base_url = f'https://{server}:3443/api/v1'
        self.short_url = f'https://{server}:3443/api'

        self.settings = Settings(
            server, config_file='settings.ini',
        )

        self.session = Auth.session(self, username, password)
        self.avatar = Avatar(self)
        self.dm_im = DmIm(self)
        self.groups = Groups(self)
        self.polls = Polls(self)
        self.rooms = Rooms(self)
        self.supergroups = SuperGroups(self)
        self.folders = Folders(self)
        self.users = Users(self)
        self.different = Different(self)
        self.chat = Chat(self)
        self.threds = Threads(self)
        self.testing = Testing(self)


__all__ = ['Client']
__version__ = '1.0.0'
__author__ = 'Eltex SC VoIP'
__description__ = 'ElphChat API Client Library'
